import importlib.metadata
import os

import typer
from dotenv import load_dotenv
from openai import OpenAI

from .actions import *
from .config import *

app = typer.Typer()
load_dotenv()

project_path = os.getcwd()

@app.command()
def version():
    """
    Display version
    """
    version_string = importlib.metadata.version('miniogre')
    print("{}".format(version_string))

    return 0

@app.command()
def readme(provider: str = "openai"):
    """
    Rewrite readme
    """

    display_figlet()
    starting_emoji()

    ogre_dir_path = config_ogre_dir(os.path.join(project_path, os.getenv("OGRE_DIR", OGRE_DIR)))
    context_contents = run_gptify(os.getcwd())
    new_readme = rewrite_readme(provider, context_contents)
    readme_path = save_readme(new_readme, ogre_dir_path)
    end_emoji()

    return 0

@app.command()
def run(
    provider: str = "openai",
    baseimage: str = "auto",
    port: str = "8001",
    dry: bool = False,
    force_requirements_generation: bool = True,
    sbom_format: str = "pip-licenses",
    no_container: bool = False,
    verbose: bool = False,
    with_readme: bool = False
):
    """
    Run full miniogre pipeline
    """

    display_figlet()
    starting_emoji()

    if baseimage == "auto":
        baseimage = config_baseimage()

    project_name = os.path.basename(project_path)

    files = list_files(project_path)
    extensions = get_extensions(files)
    counts = count_extensions(extensions)
    most_ext = determine_most_ext(counts)
    ogre_dir_path = config_ogre_dir(os.path.join(project_path, os.getenv("OGRE_DIR", OGRE_DIR)))
    if with_readme:
        context_contents = run_gptify(os.getcwd())
        new_readme = rewrite_readme(provider, context_contents)
        readme_path = save_readme(new_readme, ogre_dir_path)
    generate_requirements = config_requirements(project_path, ogre_dir_path, force_requirements_generation)
    local_requirements = extract_requirements_from_code(project_path, most_ext, generate_requirements)
    final_requirements = clean_requirements(provider, local_requirements)
    requirements_fullpath = save_requirements(final_requirements, ogre_dir_path)
    config_bashrc(project_path, ogre_dir_path, None, None, None)
    config_dockerfile(project_path, project_name, ogre_dir_path, baseimage, dry)
    create_sbom(project_name, project_path, sbom_format, verbose)
    if no_container == False:
        build_docker_image(
            os.path.join(ogre_dir_path, "Dockerfile"), project_name, verbose
        )
        spin_up_container(project_name, project_path, port)
    end_emoji()

if __name__ == "__main__":
    app()



import os
import platform
from dotenv import load_dotenv
from .constants import *


load_dotenv()

def config_ogre_dir(ogre_dir):
    """
    Check for existence of ogre_dir directory. If it exists, nothing is
    done. If it doesn't, it is created.
    """
    if not os.path.exists(ogre_dir):
        os.makedirs(ogre_dir)

    return ogre_dir

def _run_welcome(project_path, product, version, ogre_dir, date):

    original_pwd = os.getcwd()

    os.chdir(project_path)

    repo = os.popen("git remote get-url origin").read()
    author = os.popen("git log -1 --pretty=format:'%ae'").read()
    commit = os.popen("git log -1 --pretty=%h").read()
    # date = datetime.today().strftime("%Y-%m-%d-%H:%M:%S")

    os.chdir(original_pwd)

    BOILERPLATE = """
        echo REPOSITORY = {} 
        echo COMMIT = {}
        echo COMMIT_AUTHOR = {} 
        """.format(repo[:-1], commit, author)

    with open("{}/bashrc".format(ogre_dir), "a") as f:
        f.write(BOILERPLATE)
    f.close()

def config_bashrc(project_dir, ogre_dir, product, version, date):

    """
    Check for existence of bashrc. If it exists, nothing is done. If it
    doesn't, it is created.
    """

    if os.path.isfile("{}/bashrc".format(project_dir)):
        #print("   bashrc exists in {}".format(project_dir))
        os.popen("cp {}/bashrc {}/bashrc".format(project_dir, ogre_dir))

    else:
        #print("   bashrc doesn't exist. Making a new one for you.")
        with open("{}/bashrc".format(ogre_dir), "w") as f:
            f.write(BASHRC)
        f.close()
    _run_welcome(project_dir, product, version, ogre_dir, date)

    return os.path.isfile("{}/bashrc".format(ogre_dir))

def config_dockerfile(project_dir, project_name, ogre_dir,
                      baseimage, dry = False):
    """
    Check for existence of Dockerfile. If it exists, nothing is done. If it
    doesn't, a new Dockerfile is generate following the parameters in the
    config file.
    """

    REQUIREMENTS_LINE = 'RUN cat ./{}/requirements.txt | xargs -L 1 uv pip install --system; exit 0'.format(os.path.basename(ogre_dir))

    if dry:
        print(
            "Dry build -- no requirements will be installed {}".format(
                baseimage
            )
        )

        dockerfile_string = DOCKERFILE_DRY

        with open("{}/Dockerfile".format(ogre_dir), "w") as f:
            f.write(dockerfile_string.format(project_name))
        f.close()
        with open("{}/Dockerfile".format(ogre_dir), "r+") as f:
            content = f.read()
            f.seek(0, 0)
            f.write("FROM {}".format(baseimage) + content)
        f.close()
    
    else:

        if os.path.isfile("{}/Dockerfile".format(project_dir)):
            print("   Dockerfile exists in {}".format(project_dir))
            os.popen("cp {}/Dockerfile {}/Dockerfile".format(project_dir, ogre_dir))
        else:
            print(
                "   Dockerfile doesn't exist. Making a new one for you using {}".format(
                    baseimage
                )
            )

            dockerfile_string = DOCKERFILE

            with open("{}/Dockerfile".format(ogre_dir), "w") as f:
                f.write(dockerfile_string.format(project_name))
            f.close()
            with open("{}/Dockerfile".format(ogre_dir), "r+") as f:
                content = f.read()
                f.seek(0, 0)
                f.write("FROM {}".format(baseimage) + content)
                # Find last line
                f.seek(0, 2)
                #while f.read(1) != b'\n':
                #    f.seek(-2, 1)
                f.write("{}".format(REQUIREMENTS_LINE))
            f.close()

    return os.path.isfile("{}/Dockerfile".format(ogre_dir))


def config_baseimage():

    platform_machine = "{}".format(platform.machine())
    baseimage = (os.getenv("OGRE_BASEIMAGE", OGRE_BASEIMAGE)).format(platform_machine)

    return baseimage
    
def config_requirements(project_dir, ogre_dir, force=False):

    # Check if requirements.txt is in the root folder
    if os.path.isfile("{}/requirements.txt".format(project_dir)):
        if force:
            print("   requirements.txt exists in {}, but it will be OVERWRITTEN".format(project_dir))
            return True
        print("   requirements.txt already exists and it will be reused.")
        os.popen("cp {}/requirements.txt {}/requirements.txt".format(project_dir, ogre_dir))
        res = False
    else:
        print("   requirements.txt does not exist. miniogre will create one for you.")
        res = True
    return res


DOCKERFILE = """
ENV TZ=America/Chicago
WORKDIR /opt/{}
RUN ln -snf /usr/share/zoneinfo/$TZ /etc/localtime && echo $TZ > /etc/timezone
COPY . .
RUN cp ./ogre_dir/bashrc /etc/bash.bashrc
RUN chmod a+rwx /etc/bash.bashrc
RUN pip install uv pip-licenses cyclonedx-bom
"""

DOCKERFILE_DRY = """
ENV TZ=America/Chicago
WORKDIR /opt/{}
RUN ln -snf /usr/share/zoneinfo/$TZ /etc/localtime && echo $TZ > /etc/timezone
COPY . .
RUN cp ./ogre_dir/bashrc /etc/bash.bashrc
RUN chmod a+rwx /etc/bash.bashrc
RUN pip install uv pip-licenses cyclonedx-bom
"""

BASHRC = """
_python_argcomplete() {
    local IFS=$'\013'
    local SUPPRESS_SPACE=0
    if compopt +o nospace 2> /dev/null; then
        SUPPRESS_SPACE=1
    fi
    COMPREPLY=( $(IFS="$IFS" \
                  COMP_LINE="$COMP_LINE" \
                  COMP_POINT="$COMP_POINT" \
                  COMP_TYPE="$COMP_TYPE" \
                  _ARGCOMPLETE_COMP_WORDBREAKS="$COMP_WORDBREAKS" \
                  _ARGCOMPLETE=1 \
                  _ARGCOMPLETE_SUPPRESS_SPACE=$SUPPRESS_SPACE \
                  "$1" 8>&1 9>&2 1>/dev/null 2>/dev/null) )
    if [[ $? != 0 ]]; then
        unset COMPREPLY
    elif [[ $SUPPRESS_SPACE == 1 ]] && [[ "$COMPREPLY" =~ [=/:]$ ]]; then
        compopt -o nospace
    fi
}
complete -o nospace -o default -o bashdefault -F _python_argcomplete "az"

[ -z "$PS1" ] && return

export PS1="\[\e[31m\]ogre\[\e[m\] \[\e[33m\]\w\[\e[m\] > "
export TERM=xterm-256color
alias grep="grep --color=auto"
alias ls="ls --color=auto"

echo -e "\e[1;36m"
cat<<'OGRE'
  ___   __ _ _ __ ___
 / _ \ / _` | '__/ _ \ 
| (_) | (_| | | |  __/
 \___/ \__, |_|  \___|
       |___/
OGRE
echo -e "\e[0;33m"

echo "
Made by ogre.run, Inc.

Reach out to us: contact@ogre.run
"

# Turn off colors
# echo -e "\e[m"

# Aliases
alias python="python3"
"""

FILE_EXTENSIONS = {
    'python': ['.py'],
    'javascript': ['.js'],
    'c++': ['.cpp', '.cxx', '.cc', '.c', '.hpp', '.hxx', '.hh', '.h'],
    'java': ['.java'],
    'c#': ['.cs'],
    'ruby': ['.rb'],
    'php': ['.php'],
    'go': ['.go'],
    'rust': ['.rs'],
    'swift': ['.swift'],
    'kotlin': ['.kt'],
    'scala': ['.scala'],
    'clojure': ['.clj'],
    'haskell': ['.hs'],
    'erlang': ['.erl'],
    'elixir': ['.ex', '.exs'],
    'lua': ['.lua'],
    'perl': ['.pl'],
    'r': ['.r'],
    'julia': ['.jl'],
    'dart': ['.dart'],
    'typescript': ['.ts'],
    'coffeescript': ['.coffee'],
    'ocaml': ['.ml', '.mli'],
    'f#': ['.fs', '.fsi', '.fsx'],
    'scheme': ['.scm', '.ss'],
    'common lisp': ['.lisp', '.lsp'],
    'racket': ['.rkt'],
    'nim': ['.nim'],
    'crystal': ['.cr'],
    'groovy': ['.groovy'],
    'd': ['.d'],
    'fortran': ['.f', '.f90', '.f95'],
    'cobol': ['.cbl', '.cob', '.cpy'],
    'jupyter': ['.ipynb']
}

ALL_EXTENSIONS = ['.py', '.js', '.cpp', '.cxx', '.cc', '.c', '.hpp', '.hxx', '.hh', '.h', '.java', '.cs', '.rb', '.php', '.go', '.rs', '.swift', '.kt', '.scala', '.clj', '.hs', '.erl', '.ex', '.exs', '.lua', '.pl', '.r', '.jl', '.dart', '.ts', '.coffee', '.ml', '.mli', '.fs', '.fsi', '.fsx', '.scm', '.ss', '.lisp', '.lsp', '.rkt', '.nim', '.cr', '.groovy', '.d', '.f', '.f90', '.f95', '.cbl', '.cob', '.cpy', '.ipynb']

OGRE_DIR = 'ogre_dir'
OGRE_BASEIMAGE = 'ogrerun/base:ubuntu22.04-{}'

OPENAI_MODEL = 'gpt-4-turbo-preview'
OPENAI_SECRET_PROMPT = '''You are a Python requirements generator.
You should generate the contents of a Python requirements file (raw text only) taking into account the text sent by the user.
The raw text sent by the user consists of a combination of the README file contents and the source code contents.
You generate only the file contents as answer.
If the text sent by the user is invalid or is empty, just generate an empty content.
You should ignore the Python version 2 or 3.
The python package should not be included in the requirements file.
Note that some packages do not exist in the PyPi repository, they are only local, and thus shouldnt be added to the requirements.txt file.
Ignore the following Python packages: git, jittor, cuda, shihong, nvdiffrast: they do not exist on the PyPi repository.
Your output should be a raw ASCII text file.'''

OLLAMA_MODEL = 'mistral:7b'
OLLAMA_API_SERVER = 'http://localhost:11434/v1'
# OLLAMA_SECRET_PROMPT = '''You are a Python requirements generator.
# You should generate the contents of a Python requirements file (raw text only) taking into account the text sent by the user.
# The raw text sent by the user consists of a combination of the README file contents and the source code contents.
# You generate only the file contents as answer.
# If the text sent by the user is invalid or is empty, just generate an empty content.
# You should ignore the Python version 2 or 3.
# The python package should not be included in the requirements file.
# Note that some packages do not exist in the PyPi repository, they are only local, and thus shouldnt be added to the requirements.txt file.
# Ignore the following Python packages: git, jittor, cuda, shihong, nvdiffrast: they do not exist on the PyPi repository.
# Your output should be a raw ASCII text file.'''

OCTOAI_MODEL = 'mistral-7b-instruct-fp16'
OCTOAI_SECRET_PROMPT = '''You are a Python requirements generator.
You should generate the contents of a Python requirements file (raw text only) taking into account the text sent by the user.
The raw text sent by the user consists of a combination of the README file contents and the source code contents.
If the text sent by the user is invalid or is empty, just generate an empty content.
You should ignore the Python version 2 or 3.
The python package should not be included in the requirements file.
Some packages do not exist in the PyPi repository, they are only local, and thus shouldnt be added to the requirements.txt file.
Ignore the following Python packages: git, jittor, cuda, shihong, nvdiffrast: they do not exist on the PyPi repository.
Your output should be a raw ASCII text file.
Do not return parts of the text sent by the user. We just want the requirements list.
Only return the list of requirements. No other text like the filename at the top of the response or symbols are allowed.'''

REWRITE_README_PROMPT = '''You are a specialist in understanding and explaining source code. 
You are also a specialist in writing clear documentation (e.g README files) that helps people to understand the source code.
Your task is to take a text input containing the current README and the code and use it to write an updated version of the README file.
The README file should highlight the actual requirements that need to be installed.'''

GROQ_MODEL = 'mixtral-8x7b-32768'
GROQ_SECRET_PROMPT = '''You are a Python requirements generator.
You should generate the contents of a Python requirements file (raw text only) taking into account the text sent by the user.
The raw text sent by the user consists of a combination of the README file contents and the source code contents.
If the text sent by the user is invalid or is empty, just generate an empty content.
You should ignore the Python version 2 or 3.
The python package should not be included in the requirements file.
Some packages do not exist in the PyPi repository, they are only local, and thus shouldnt be added to the requirements.txt file.
Ignore the following Python packages: git, jittor, cuda, shihong, nvdiffrast: they do not exist on the PyPi repository.
Your output should be a raw ASCII text file containg only the list of requirements. Do not return sentences. 
Do not return parts of the text sent by the user. We just want the requirements list.'''

MISTRAL_MODEL = 'mistral-large-latest'
# MISTRAL_SECRET_PROMPT = '''You are a Python requirements generator.
# You should generate the contents of a Python requirements file (raw text only) taking into account the text sent by the user.
# The raw text sent by the user consists of a combination of the README file contents and the source code contents.
# You generate only the file contents as answer.
# If the text sent by the user is invalid or is empty, just generate an empty content.
# You should ignore the Python version 2 or 3.
# The python package should not be included in the requirements file.
# Note that some packages do not exist in the PyPi repository, they are only local, and thus shouldnt be added to the requirements.txt file.
# Ignore the following Python packages: git, jittor, cuda, shihong, nvdiffrast: they do not exist on the PyPi repository.
# Your output should be a raw ASCII text file.'''

CLEAN_REQUIREMENTS_SECRET_PROMPT='''Here is the content of a requirements.txt file. 
It is a list of Python libraries to be installed in the a Python environment. 
Not all entries in the list are actual Python libraries available on the PyPi repository. 
Identify those that arent available on PyPi and remove them from the list.
Get the remaining entries (those that are available on PypI) and generate an updated list of requirements.
If any of the packages in the updated list is represented by a name under which it can not be found on PyPi, change the name to the one that is available on PyPi.
For example, the library `PIL` is not available under that name in PyPi, but `Pillow` is.
Do not explain how the new list is generated. Do not provide any context related to how you proceeded.
Your response must contain only the list of packages to be installed.
Remove any blank lines.'''

GROQ_CLEAN_REQUIREMENTS_SECRET_PROMPT='''Here is the content of a requirements.txt file. 
It is a list of Python libraries to be installed in the a Python environment. 
Not all entries in the list are actual Python libraries available on the PyPi repository. 
Identify those that arent available on PyPi and remove them from the list.
Get the remaining entries (those that are available on PypI) and generate an updated list of requirements.
If any of the packages in the updated list is represented by a name under which it can not be found on PyPi, change the name to the one that is available on PyPi.
For example, the library `PIL` is not available under that name in PyPi, but `Pillow` is.
No explanation notes on your reasoning are allowed. Nobody wants to know which packages were removed. Your response must contain only the list of packages to be installed.
Remove any blank lines.'''



import os
import ast
import emoji
import platform
import subprocess
from openai import OpenAI
from octoai.client import Client as OctoAiClient
from groq.cloud.core import Completion
from mistralai.client import MistralClient
from mistralai.models.chat_completion import ChatMessage
from pyfiglet import Figlet
from rich import print as rprint
from yaspin import yaspin
from .constants import *


def starting_emoji():
    print(emoji.emojize(':ogre: Starting miniogre...'))

def end_emoji():
    print(emoji.emojize(':hourglass_done: Done.'))

def build_emoji():
    print(emoji.emojize(':spouting_whale: Building Docker image...'))

def spinup_emoji():
    print(emoji.emojize(':rocket: Spinning up container...'))

def requirements_emoji():
    print(emoji.emojize(':thinking_face: Generating requirements...'))

def cleaning_requirements_emoji():
    print(emoji.emojize(':cooking: Refining...'))

def generate_context_emoji():
    print(emoji.emojize(':magnifying_glass_tilted_left: Generating context...'))

def readme_emoji():
    print(emoji.emojize(':notebook: Generating new README.md...'))

def list_files(project_path):
    # List all files in current directory and subdirectories
    files = []
    for root, dirs, filenames in os.walk(project_path):
        if '.git' in dirs:
            dirs.remove('.git')
        if '__pycache__' in dirs:
            dirs.remove('__pycache__')
        for filename in filenames:
            files.append(os.path.join(root, filename))
    return files

# Get file extensions
def get_extensions(files):
    extensions = [os.path.splitext(f)[1] for f in files]
    return extensions

# Count file extensions
def count_extensions(extensions):
    counts = {}
    for ext in extensions:
        if ext in counts:
            counts[ext] += 1
        else:
            counts[ext] = 1
    return counts

# Get most prevalent extension for code files
def determine_most_ext(counts):
    #TODO: implement logic to determine the codebase.    
    return '.py'

def find_readme(project_path):
    files = list_files(project_path)
    readme_files = [f for f in files if os.path.basename(f).lower().startswith('readme')]

    if readme_files:
        return readme_files[0]
    else:
        return None

def read_file_contents(path_to_file):
    if path_to_file == None:
        return ''
    else:
        if os.path.exists(path_to_file):
            with open(path_to_file, 'r') as f:
                contents = f.read()
            return contents
        else:
            return ''

def conform_to_pep8(filename):

    pep8_cmd = 'autopep8 --in-place --aggressive {}'.format(filename)
    
    p = subprocess.Popen(pep8_cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    (out, err) = p.communicate()
    p_status = p.wait()

    return 0

def extract_external_imports(code):

    external_imports = []
    
    tree = ast.parse(code)
    
    for node in ast.walk(tree):
        if isinstance(node, ast.Import):
            for name in node.names:
                if not name.name.startswith('.'):
                    external_imports.append(name.name)
        elif isinstance(node, ast.ImportFrom):
            if not node.module.startswith('.'):
                external_imports.append(node.module)
   
    return external_imports

def extract_requirements_from_code(project_path, ext, generate = True):

    requirements_emoji()

    if generate:
        files = list_files(project_path)
        matching = [f for f in files if os.path.splitext(f)[1] == ext]

        external_imports = []
        for filename in matching:
            i = 0
            while i < 2:
                with open(os.path.join(project_path, filename), 'r') as readfile:
                    content = readfile.read()
                    try:
                        external_imports.append(extract_external_imports(content))
                        i = 2
                    except Exception as e:
                        if i == 1:
                            i = 2
                            print(e)
                            print("External imports extraction failed for file {}: {}".format(filename, Exception))
                        else:
                            conform_to_pep8(filename)
                            i += 1
                
        external_imports = [imp.split('.')[0] for sublist in external_imports for imp in sublist]
        external_imports = list(set(external_imports))

        requirements = '\n'.join(external_imports)
    else:
        with open('{}/requirements.txt'.format(os.getenv('OGRE_DIR', OGRE_DIR)), 'r') as f:
            requirements = f.read()
    return requirements



def append_files_with_ext(project_path, ext, limit, output_file):
    files = list_files(project_path)
    matching = [f for f in files if os.path.splitext(f)[1] == ext]
    
    if limit < len(matching):
        matching = matching[:limit]
        
    contents = ''
    with open(output_file, 'a') as outfile:
        for filename in matching:
            with open(os.path.join(project_path, filename), 'r') as readfile:
                contents += readfile.read()

    return contents

def generate_context_file(readme_text, source_text, output_file):
    """
    Generates the context file by appending the README and the source code text. 
    The content of this file will be used to extract the dependencies.
    """
    out_text = readme_text + source_text
    
    with open(output_file, 'w') as out:
        out.write(out_text)
    
    with open(output_file, 'r') as f:
        return f.read()

def read_context(path_to_context_file):
    with open(path_to_context_file, 'r') as f:
        contents = f.read()
    return contents

def extract_requirements(provider, contents):
    requirements_emoji()
    if provider == 'openai':
        res = extract_requirements_openai(contents)
    elif provider == 'octoai':
        res = extract_requirements_octoai(contents)
    elif provider == 'groq':
        res = extract_requirements_groq(contents)
    return res

def extract_requirements_openai(contents):
    model = os.getenv('OPENAI_MODEL', OPENAI_MODEL)
    prompt = os.getenv('OPENAI_SECRET_PROMPT', OPENAI_SECRET_PROMPT)
    client = OpenAI()
    completion = client.chat.completions.create(
                  model=model,
                  messages=[
                      {"role": "system", "content": prompt},
                      {"role": "user", "content": contents}
                  ]
              )
    requirements = completion.choices[0].message.content
    
    return requirements

def extract_requirements_octoai(contents):
    model = os.getenv('OCTOAI_MODEL', OCTOAI_MODEL)
    prompt = os.getenv('OCTOAI_SECRET_PROMPT', OCTOAI_SECRET_PROMPT)
    client = OctoAiClient()

    completion = client.chat.completions.create(
    messages=[
            {
                "role": "system",
                "content": prompt
            },
            {
                "role": "user",
                "content": contents
            }
        ],
    model=model,
    max_tokens=20000,
    presence_penalty=0,
    temperature=0.1,
    top_p=0.9)

    requirements = completion.choices[0].message.content

    return requirements

def extract_requirements_groq(contents):
    prompt = os.getenv('GROQ_SECRET_PROMPT', GROQ_SECRET_PROMPT)
    with Completion() as completion:
        full_prompt = prompt + " " + contents
        response, id, stats = completion.send_prompt("llama2-70b-4096", user_prompt=full_prompt)
        if response != "":
            print(f"\nPrompt: {prompt}\n")
            print(f"Request ID: {id}")
            print(f"Output:\n {response}\n")
            print(f"Stats:\n {stats}\n")
    return response

def clean_requirements(provider, requirements):
    cleaning_requirements_emoji()
    if provider == 'openai':
        res = clean_requirements_openai(requirements)
    elif provider == 'ollama':
        res = clean_requirements_ollama(requirements)
    elif provider == 'octoai':
        res = clean_requirements_octoai(requirements)
    elif provider == 'groq':
        res = clean_requirements_groq(requirements)
    elif provider == 'mistral':
        res = clean_requirements_mistral(requirements)
    return res

def clean_requirements_ollama(requirements):
    model = os.getenv('OLLAMA_MODEL', OLLAMA_MODEL)
    prompt = os.getenv('CLEAN_REQUIREMENTS_SECRET_PROMPT', CLEAN_REQUIREMENTS_SECRET_PROMPT)
    api_server = os.getenv('OLLAMA_API_SERVER', OLLAMA_API_SERVER)
    # print(f"{api_server=} {model=} {prompt=}")
    client = OpenAI(base_url=api_server, api_key='ollama')
    completion = client.chat.completions.create(
                  model=model,
                  messages=[
                      {"role": "system", "content": prompt},
                      {"role": "user", "content": requirements}
                  ]
              )
    requirements = completion.choices[0].message.content

    return requirements

def clean_requirements_openai(requirements):
    model = os.getenv('OPENAI_MODEL', OPENAI_MODEL)
    prompt = os.getenv('CLEAN_REQUIREMENTS_SECRET_PROMPT', CLEAN_REQUIREMENTS_SECRET_PROMPT)
    # print(f"{model=} {prompt=}")
    client = OpenAI()
    completion = client.chat.completions.create(
                  model=model,
                  messages=[
                      {"role": "system", "content": prompt},
                      {"role": "user", "content": requirements}
                  ]
              )
    requirements = completion.choices[0].message.content

    return requirements

def clean_requirements_mistral(requirements):
    model = os.getenv('MISTRAL_MODEL', MISTRAL_MODEL)
    prompt = os.getenv('CLEAN_REQUIREMENTS_SECRET_PROMPT', CLEAN_REQUIREMENTS_SECRET_PROMPT)
    api_key = os.environ["MISTRAL_API_KEY"]
    client = MistralClient(api_key=api_key)
    content = prompt + '\n' + requirements
    messages = [ChatMessage(role="user", content=content)]

    # No streaming
    chat_response = client.chat(
        model=model,
        messages=messages,
    )
    requirements = chat_response.choices[0].message.content

    return requirements

def clean_requirements_groq(requirements):
    model = os.getenv('GROQ_MODEL', GROQ_MODEL)
    prompt = os.getenv('GROQ_CLEAN_REQUIREMENTS_SECRET_PROMPT', GROQ_CLEAN_REQUIREMENTS_SECRET_PROMPT)
    with Completion() as completion:
        full_prompt = prompt + " " + requirements
        response, id, stats = completion.send_prompt(model, user_prompt=full_prompt)
        
    return response

def clean_requirements_octoai(requirements):
    model = os.getenv('OCTOAI_MODEL', OCTOAI_MODEL)
    prompt = os.getenv('CLEAN_REQUIREMENTS_SECRET_PROMPT', CLEAN_REQUIREMENTS_SECRET_PROMPT)
    client = OctoAiClient()

    completion = client.chat.completions.create(
        messages=[
                {
                    "role": "system",
                    "content": prompt
                },
                {
                    "role": "user",
                    "content": requirements
                }
            ],
        model=model,
        max_tokens=20000,
        presence_penalty=0,
        temperature=0.1,
        top_p=0.9)

    requirements = completion.choices[0].message.content

    return requirements

def save_requirements(requirements, ogre_dir_path):
    requirements_fullpath = os.path.join(ogre_dir_path, 'requirements.txt')
    with open(requirements_fullpath, 'w') as f:
        f.write(requirements)
    return requirements_fullpath 

def rewrite_readme(provider, readme):
    readme_emoji()
    
    if provider == 'openai':
        res = rewrite_readme_openai(readme)
    elif provider == 'ollama':
        res = rewrite_readme_ollama(readme)
    elif provider == 'octoai':
        res = rewrite_readme_octoai(readme)
    elif provider == 'groq':
        res = rewrite_readme_groq(readme)
    elif provider == 'mistral':
        res = rewrite_readme_mistral(readme)
    return res

def rewrite_readme_openai(readme):
    model = os.getenv('OPENAI_MODEL', OPENAI_MODEL)
    prompt = os.getenv('REWRITE_README_PROMPT', REWRITE_README_PROMPT)
    client = OpenAI()
    if 'OPENAI_API_KEY' not in os.environ:
        raise EnvironmentError("OPENAI_API_KEY environment variable not defined")
    
    try:
        completion = client.chat.completions.create(
                        model=model,
                        messages=[
                            {"role": "system", "content": prompt},
                            {"role": "user", "content": readme}
                        ]
                    )
        # print(completion)
        new_readme = completion.choices[0].message.content
    except Exception as e:
        print(e)

    return new_readme

def rewrite_readme_ollama(readme):
    model = os.getenv('OLLAMA_MODEL', OLLAMA_MODEL)
    prompt = os.getenv('REWRITE_README_PROMPT', REWRITE_README_PROMPT)
    api_server = os.getenv('OLLAMA_API_SERVER', OLLAMA_API_SERVER)
    # print(f"{api_server=} {model=} {prompt=}")
    client = OpenAI(base_url=api_server, api_key='ollama')
    completion = client.chat.completions.create(
                  model=model,
                  messages=[
                      {"role": "system", "content": prompt},
                      {"role": "user", "content": readme}
                  ]
              )
    new_readme = completion.choices[0].message.content

    return new_readme

def rewrite_readme_octoai(readme):
    raise NotImplementedError("rewrite_readme_octoai is not implemented.")

def rewrite_readme_groq(readme):
    raise NotImplementedError("rewrite_readme_groq is not implemented.")

def rewrite_readme_mistral(readme):
    model = os.getenv('MISTRAL_MODEL', MISTRAL_MODEL)
    prompt = os.getenv('REWRITE_README_PROMPT', REWRITE_README_PROMPT)
    api_key = os.environ["MISTRAL_API_KEY"]
    client = MistralClient(api_key=api_key)
    content = prompt + '\n' + readme
    messages = [ChatMessage(role="user", content=content)]
    
    # No streaming
    chat_response = client.chat(
        model=model,
        messages=messages,
    )
    new_readme = chat_response.choices[0].message.content

    return new_readme

def save_readme(readme, ogre_dir_path):
    readme_fullpath = os.path.join(ogre_dir_path, 'README.md')
    with open(readme_fullpath, 'w') as f:
        f.write(readme)
    return readme_fullpath 

def build_docker_image(dockerfile, image_name, verbose = False):
    # build docker image
    
    platform_name = "linux/{}".format(platform.machine())
    image_name = "miniogre/{}:{}".format(image_name.lower(), "latest")

    build_emoji()
    print("   platform = {}".format(platform_name)) 
    print("   image name = {}".format(image_name))
    
    if verbose:
        stderr = None
        progress = 'plain'
    else:
        stderr = subprocess.PIPE
        progress = 'auto'

    build_cmd = (
        "DOCKER_BUILDKIT=1 docker buildx build --no-cache --load --progress={} --platform {} -t {} -f {} .".format(
            progress, platform_name, image_name, dockerfile
        )
    )
    print("   build command = {}".format(build_cmd))

    if verbose:
        p = subprocess.Popen(build_cmd, stdout=subprocess.PIPE, stderr=stderr, shell=True)
        (out, err) = p.communicate()
        p_status = p.wait()
    else:
        with yaspin().aesthetic as sp:
            sp.text = "generating ogre environment" 
            p = subprocess.Popen(build_cmd, stdout=subprocess.PIPE, stderr=stderr, shell=True)
            (out, err) = p.communicate()
            p_status = p.wait()
    return out

def spin_up_container(image_name, project_path, port):
    # spin up container
    spinup_emoji()

    project_name = image_name
    container_name = "miniogre-{}".format(image_name.lower())
    image_name = "miniogre/{}:{}".format(image_name.lower(), "latest")
    #cmd = "uv venv; source .venv/bin/activate; cat ./{}/requirements.txt | xargs -L 1 uv pip install; exit 0;"
    #cmd = "cat ./ogre_dir/requirements.txt | xargs -L 1 uv pip install; exit 0"
    spin_up_cmd = (
        "docker run -it --rm -v {}:/opt/{} -p {}:{} --name {} {}".format(project_path, project_name, port, port, container_name, image_name)  
    )

    print("   spin up command = {}".format(spin_up_cmd))
    subprocess.call(spin_up_cmd.split())
    #p = subprocess.Popen(spin_up_cmd, stdout=subprocess.PIPE, shell=True)
    #(out, err) = p.communicate()
    #p_status = p.wait()

    return 0

def create_sbom(image_name, project_path, format, verbose = False):
    # Create SBOM from inside the container
    print(emoji.emojize(':desktop_computer:  Generating SBOM...'))

    project_name = image_name
    container_name = "miniogre-{}".format(image_name.lower())
    image_name = "miniogre/{}:{}".format(image_name.lower(), "latest")

    if format == 'cyclonedx':
        sbom_format_cmd = "cyclonedx-py requirements ./ogre_dir/requirements.txt &> ./ogre_dir/sbom.json"
    elif format == 'pip-licenses':
        sbom_format_cmd = "pip-licenses --with-authors --with-maintainers --with-urls --with-description -l --format json --output-file ./ogre_dir/sbom.json"

    # sbom_cmd = (
    #     "   docker run -d --rm -v {}:/opt/{} --name {}_sbom {} bash -c '{}; wait'".format(project_path, project_name, container_name, image_name, sbom_format_cmd)  
    # )
    sbom_cmd = sbom_format_cmd
    
    if verbose:
        stderr = None
        print(sbom_cmd)
    else:
        stderr = subprocess.PIPE
    p = subprocess.Popen(sbom_cmd, stdout=subprocess.PIPE, stderr=stderr, shell=True)
    p.communicate()
    p.wait()

    return 0
    
def display_figlet():
    # Display Ogre figlet
    f = Figlet(font="slant")
    # Get version
    # ogre_version = pkg_resources.get_distribution('miniogre').version
    rprint("[cyan] {} [/cyan]".format(f.renderText("miniogre")))
    rprint("[blue bold]miniogre - {}[/blue bold]".format("https://ogre.run"))
    print("\n")

# def create_virtualenv(requirements, python_version):
  
#     env_name = 'miniogre-env'

#     venv_cmd = "python -m venv {}".format(env_name)
#     # venv_activate_cmd = 'source {}/bin/activate'.format(env_name) 

#     #os.popen(venv_cmd)
#     #os.popen(venv_activate_cmd)
#     p = subprocess.Popen(venv_cmd, stdout=subprocess.PIPE, shell=True)
#     (out, err) = p.communicate()
#     p_status = p.wait()

#     venv_activate_cmd = 'source {}/bin/activate'.format(env_name)
#     p = subprocess.Popen(venv_activate_cmd, stdout=subprocess.PIPE, shell=True)
#     (out, err) = p.communicate()
#     p_status = p.wait()

#     pip_cmd = '{}/bin/pip'.format(env_name)
#     with open(requirements) as f:
#         requirements_list = []
#         for line in f:
#             requirements_list.append(line.strip('\n'))

#         pip_cmd = 'pip' 
#         for req in requirements_list:
#             print(req)
#             input()
#             subprocess.call([pip_cmd, 'install', req.strip()])

def run_gptify(repo_path):

    generate_context_emoji()
    
    if not os.path.exists("ogre_dir"):
        os.mkdir("ogre_dir")

    gptrepo_cmd = 'gptify {} --output "ogre_dir/gptify_output.txt"'.format(repo_path)
    p = subprocess.Popen(gptrepo_cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    (out, err) = p.communicate()
    p_status = p.wait()

    with open('ogre_dir/gptify_output.txt', 'r') as f:
        return f.read()

def cleanup():
    """
    Delete unnecessary files
    """
    if os.path.exists('ogre_dir/gptify_output.txt'):
        os.remove('ogre_dir/gptify_output.txt')
